@@ -9,26 +9,34 @@ import subprocess
 import socket
 import ctypes
 import uuid
-
-from binarycpython.utils.functions import temp_dir, remove_file
+from typing import Union, Tuple, Optional
+from binarycpython.utils.functions import temp_dir, remove_file, verbose_print
 
 
-def autogen_C_logging_code(logging_dict, verbose=0):
+def autogen_C_logging_code(logging_dict: dict, verbose: int = 0) -> Optional[str]:
     """
     Function that autogenerates PRINTF statements for binaryc.
     Input is a dictionary where the key is the header of that logging line
     and items which are lists of parameters that will be put in that logging line
 
-    Example::
-        {'MY_STELLAR_DATA':
-            [
-                'model.time',
-                'star[0].mass',
-                'model.probability',
-                'model.dt'
-            ]
-        }
+    Example:
+        input dictionary should look like this::
 
+            {'MY_STELLAR_DATA':
+                [
+                    'model.time',
+                    'star[0].mass',
+                    'model.probability',
+                    'model.dt'
+                ]
+            }
+
+    Args:
+        logging_dict: Dictionary containing lists of parameters that binary_c has to output. The keys are used by binary_c as start of the sentence.
+        verbose: Level of verbosity. Defaults to zero if not set explicilty.
+
+    Returns:
+        string containing C printf statement built to output the parameters given as input.
     """
 
     # Check if the input is of the correct form
@@ -37,6 +45,7 @@ def autogen_C_logging_code(logging_dict, verbose=0):
         return None
 
     code = ""
+
     # Loop over dict keys
     for key in logging_dict:
         if verbose > 0:
@@ -65,19 +74,59 @@ def autogen_C_logging_code(logging_dict, verbose=0):
             print(
                 "Error: please use a list for the list of parameters that you want to have logged"
             )
+            return None
     code = code.strip()
 
     return code
 
 
 ####################################################################################
-def binary_c_log_code(code, verbose=0):
+def binary_c_log_code(code: str, verbose: int = 0) -> str:
     """
     Function to construct the code to construct the custom logging function
+
+    Example:
+        Code to log and terminate evolution when the primary star becomes a NS::
+
+            if(stardata->star[0].stellar_type>=NS)
+            {
+                if (stardata->model.time < stardata->model.max_evolution_time)
+                {
+                    Printf("EXAMPLE_LOG_CO %30.12e %g %g %g %g %d %d\\n",
+                        //
+                        stardata->model.time, // 1
+
+                        stardata->star[0].mass, //2
+                        stardata->previous_stardata->star[0].mass, //3
+
+                        stardata->star[0].radius, //4
+                        stardata->previous_stardata->star[0].radius, //5
+
+                        stardata->star[0].stellar_type, //6
+                        stardata->previous_stardata->star[0].stellar_type //7
+                  );
+                };
+                /* Kill the simulation to save time */
+                stardata->model.max_evolution_time = stardata->model.time - stardata->model.dtm;
+            };
+
+    Args:
+        code: Exact c-statement to output information in binary_c. Can be wrapped in logical statements.
+        verbose: Level of verbosity. Defaults to zero if not set explicilty.
+
+    Returns:
+        string containing the custom logging code. This includes all the includes and other definitions. This code will be used as the shared library
     """
 
-    if verbose > 0:
-        print("Creating the code for the shared library for the custom logging")
+    verbose_print(
+        "Creating the code for the shared library for the custom logging", verbose, 0
+    )
+
+    if not "Printf" in code:
+        print(
+            "Error: There has to be at least a printf statement in the provided code. Aborting"
+        )
+        return None
 
     # Create code
     custom_logging_function_string = """\
@@ -108,11 +157,17 @@ void binary_c_API_function custom_output_function(struct stardata_t * stardata)
     return textwrap.dedent(custom_logging_function_string)
 
 
-def binary_c_write_log_code(code, filename, verbose=0):
+def binary_c_write_log_code(code: str, filename: str, verbose: int = 0) -> None:
     """
     Function to write the generated logging code to a file
+
+    Args:
+        code: string containing the custom logging code to write to a file.
+        filename: target filename.
+        verbose: Level of verbosity. Defaults to zero if not set explicilty.
     """
 
+    # TODO: change this. I don't like the cwd
     cwd = os.getcwd()
     filePath = os.path.join(cwd, filename)
 
@@ -127,9 +182,16 @@ def binary_c_write_log_code(code, filename, verbose=0):
         file.write(code)
 
 
-def from_binary_c_config(config_file, flag):
+def from_binary_c_config(config_file: str, flag: str) -> str:
     """
     Function to run the binaryc_config command with flags
+
+    Args:
+        config_file: binary_c-config filepath TODO: change the name of this
+        flag: flag used in the binary_c-config call.
+
+    Returns:
+        returns the result of <binary_c-config> <flag>
     """
 
     res = subprocess.check_output(
@@ -139,22 +201,25 @@ def from_binary_c_config(config_file, flag):
     )
 
     # convert and chop off newline
-    res = res.decode("utf").rstrip()
+    res = res.decode("utf-8").rstrip()
+
     return res
 
 
-def return_compilation_dict(verbose=0):
+def return_compilation_dict(verbose: int = 0) -> dict:
     """
     Function to build the compile command for the shared library
 
-    inspired by binary_c_inline_config command in perl
+    Inspired by binary_c_inline_config command in perl
 
     TODO: this function still has some cleaning up to do wrt default values for the compile command
     # https://developers.redhat.com/blog/2018/03/21/compiler-and-linker-flags-gcc/
 
+    Args:
+        verbose: Level of verbosity. Defaults to zero if not set explicilty.
 
-    returns:
-     - string containing the command to build the shared library
+    Returns:
+        string containing the command to build the shared library
     """
 
     if verbose > 0:
@@ -166,7 +231,6 @@ def return_compilation_dict(verbose=0):
     if BINARY_C_DIR:
         BINARY_C_CONFIG = os.path.join(BINARY_C_DIR, "binary_c-config")
         BINARY_C_SRC_DIR = os.path.join(BINARY_C_DIR, "src")
-        # TODO: build in check to see whether the file exists
     else:
         raise NameError("Envvar BINARY_C doesnt exist")
 
@@ -240,10 +304,20 @@ def return_compilation_dict(verbose=0):
     return {"cc": cc, "ld": ld, "ccflags": ccflags, "libs": libs, "inc": inc}
 
 
-def compile_shared_lib(code, sourcefile_name, outfile_name, verbose=0):
+def compile_shared_lib(
+    code: str, sourcefile_name: str, outfile_name: str, verbose: int = 0
+) -> None:
     """
     Function to write the custom logging code to a file and then compile it.
+
     TODO: nicely put in the -fPIC
+    TODO: consider returning a status
+
+    Args:
+        code: string containing the custom logging code
+        sourcefile_name: name of the file that will contain the code
+        outfile_name: name of the file that will be the shared library
+        verbose: Level of verbosity. Defaults to zero if not set explicilty.
     """
 
     # Write code to file
@@ -272,8 +346,6 @@ def compile_shared_lib(code, sourcefile_name, outfile_name, verbose=0):
 
     # Execute compilation and create the library
     if verbose > 0:
-        # BINARY_C_DIR = os.getenv("BINARY_C")
-        # BINARY_C_SRC_DIR = os.path.join(BINARY_C_DIR, "src")
         print(
             "Building shared library for custom logging with (binary_c.h) on {}\n".format(
                 socket.gethostname()
@@ -291,12 +363,21 @@ def compile_shared_lib(code, sourcefile_name, outfile_name, verbose=0):
             print("Output of compilation command:\n{}".format(res))
 
 
-def create_and_load_logging_function(custom_logging_code, verbose=0):
+def create_and_load_logging_function(
+    custom_logging_code: str, verbose: int = 0
+) -> Tuple[int, str]:
     """
     Function to automatically compile the shared library with the given
-    custom logging code and load it with ctypes
+    custom logging code and load it with ctypes.
 
-    returns:
+    This function is more or less the main function of this module and unless you know what you're doing with the other functions
+    I recommend using this in function in combination with a function that generates the exact code (like :meth:`~binarycpython.utils.custom_logging_code.binary_c_log_code`)
+
+    Args:
+        custom_logging_code: string containing the custom logging code
+        verbose: Level of verbosity. Defaults to zero if not set explicilty.
+
+    Returns:
         memory adress of the custom logging function in a int type.
     """
 
@@ -331,6 +412,14 @@ def create_and_load_logging_function(custom_logging_code, verbose=0):
         libcustom_logging.custom_output_function, ctypes.c_void_p
     ).value
 
+    if not isinstance(func_memaddr, int):
+        print(
+            "Something went wrong. The memory adress returned by the ctypes.cast is not an integer. It has the value {}".format(
+                func_memaddr
+            )
+        )
+        raise ValueError
+
     if verbose > 0:
         print(
             "loaded shared library for custom logging. \
