@@ -1,11 +1,23 @@
 """
-Module that contains the default options for the population gric code.
+Module that contains the default options for the population grid code along with the description for these options, in the form of dictionaries: 
+    - grid_options_defaults_dict: dictionary containing the default values for all the options
+    - grid_options_descriptions: dictionary containing the description for these options.
+
+There are several other functions in this module, mostly to generate help texts or documents:
+    - grid_options_help: interactive function for the user to get descriptions for options 
+    - grid_options_description_checker: function that checks that checks which options have a description.
+    - write_grid_options_to_rst_file: function to generate the .rst document for the docs
+
+With this its also possible to automatically generate a document containing all the setting names + descriptions. 
+
+All the options starting with _ should not be changed by the user except when you really know what you're doing (which is probably hacking the code :P)
 """
 
 import os
 
 from binarycpython.utils.custom_logging_functions import temp_dir
 
+# Options dict
 grid_options_defaults_dict = {
     ##########################
     # general (or unordered..)
@@ -14,9 +26,9 @@ grid_options_defaults_dict = {
     "binary": 0,  # FLag on whether the systems are binary systems or single systems.
     "parse_function": None,  # FUnction to parse the output with.
     "tmp_dir": temp_dir(),  # Setting the temp dir of the program
-    "main_pid": -1,  # Placeholder for the main process id of the run.
+    "_main_pid": -1,  # Placeholder for the main process id of the run.
     # "output_dir":
-    "commandline_input": "",
+    "_commandline_input": "",
     ##########################
     # Execution log:
     ##########################
@@ -27,16 +39,16 @@ grid_options_defaults_dict = {
     ##########################
     # binary_c files
     ##########################
-    "binary_c_executable": os.path.join(
+    "_binary_c_executable": os.path.join(
         os.environ["BINARY_C"], "binary_c"
     ),  # TODO: make this more robust
-    "binary_c_shared_library": os.path.join(
+    "_binary_c_shared_library": os.path.join(
         os.environ["BINARY_C"], "src", "libbinary_c.so"
     ),  # TODO: make this more robust
-    "binary_c_config_executable": os.path.join(
+    "_binary_c_config_executable": os.path.join(
         os.environ["BINARY_C"], "binary_c-config"
     ),  # TODO: make this more robust
-    "binary_c_dir": os.environ["BINARYC_DIR"],
+    "_binary_c_dir": os.environ["BINARYC_DIR"],
     ##########################
     # Custom logging
     ##########################
@@ -45,49 +57,52 @@ grid_options_defaults_dict = {
     # This will get parsed by autogen_C_logging_code in custom_loggion_functions.py
     "C_logging_code": None,  # Should contain a string which holds the logging code.
     "custom_logging_func_memaddr": -1,  # Contains the custom_logging functions memory address
-    "custom_logging_shared_library_file": None,
+    "_custom_logging_shared_library_file": None,  # file containing the .so file
     ##########################
     # Store pre-loading:
     ##########################
-    "store_memaddr": -1,  # Contains the store object memory adress, useful for preloading.
+    "_store_memaddr": -1,  # Contains the store object memory adress, useful for preloading.
     # defaults to -1 and isnt used if thats the default then.
     ##########################
     # Log args: logging of arguments
     ##########################
-    "log_args": 0,  #
-    "log_args_dir": "/tmp/",
+    "log_args": 0,  # unused
+    "log_args_dir": "/tmp/",  # unused
     ##########################
     # Population evolution
     ##########################
     ## General
     "evolution_type": "mp",  # Flag for type of population evolution
-    "evolution_type_options": [
+    "_evolution_type_options": [
         "mp",
         "linear",
     ],  # available choices for type of population evolution
-    "system_generator": None,  # value that holds the function that generates the system
+    "_system_generator": None,  # value that holds the function that generates the system
     # (result of building the grid script)
     "population_type": "grid",  #
-    "population_type_options": [
+    "_population_type_options": [
         "grid",
         "source_file",
-    ],  # Available choices for type of population generation # TODO: fill later with monte carlo etc
+    ],  # Available choices for type of population generation. Unused for now. # TODO: fill later with monte carlo etc
     "source_file_filename": None,  # filename for the source
-    "count": 0,  # total count of systems
-    "probtot": 0,  # total probability
-    "weight": 1.0,  # weighting for the probability
-    "repeat": 1.0,  # number of times to repeat each system (probability is adjusted to be 1/repeat)
-    "results_per_worker": {},  # dict which can store info per worker. meh. doesnt work properly
-    "start_time_evolution": 0,  # Start time of the grid
-    "end_time_evolution": 0,  # end time of the grid
-    "error": 0,  # error?
-    "failed_count": 0,  # amt of failed systems
-    "failed_prob": 0,  # Summed probability of failed systems
-    "id": 0,  # Random id of this grid/population run,
-    "modulo": 1,  # run modulo n of the grid.
+    "_count": 0,  # count of systems
+    "_total_starcount": 0,  # Total count of systems in this generator
+    "_probtot": 0,  # total probability
+    "_weight": 1.0,  # weighting for the probability
+    "_repeat": 1.0,  # number of times to repeat each system (probability is adjusted to be 1/repeat)
+    "results": {},  # dict to store the results. Every process fills this on its own and then it will be joined later
+    "_start_time_evolution": 0,  # Start time of the grid
+    "_end_time_evolution": 0,  # end time of the grid
+    "_errors_found": False,  # Flag whether there are any errors from binary_c
+    "_errors_exceeded": False,  # Flag whether the amt of errors have exceeded the limit
+    "_failed_count": 0,  # amt of failed systems
+    "_failed_prob": 0,  # Summed probability of failed systems
+    "failed_systems_threshold": 20,  # Maximum failed systems per process allowed to fail before the process stops logging the failing systems.
+    "_failed_systems_error_codes": [],  # List to store the unique error codes
+    "_population_id": 0,  # Random id of this grid/population run, Unique code for the population. Should be set only once by the controller process.
+    "modulo": 1,  # run modulo n of the grid. #TODO: fix this
     ## Grid type evolution
-    "grid_variables": {},  # grid variables
-    "grid_code": None,  # literal grid code: contains the whole script that'll be written to a file
+    "_grid_variables": {},  # grid variables
     "gridcode_filename": None,  # filename of gridcode
     ## Monte carlo type evolution
     # TODO: make MC options
@@ -407,3 +422,203 @@ grid_options_defaults_dict = {
     # # stop and start a grid
     # starting_snapshot_file=>undef,
 }
+
+# Grid containing the descriptions of the options # TODO: add input types for all of them
+grid_options_descriptions = {
+    "tmp_dir": "Directory where certain types of output are stored. The grid code is stored in that directory, as well as the custom logging libraries. Log files and other diagnostics will usually be written to this location, unless specified otherwise",  # TODO: improve this
+    "_binary_c_dir": "Director where binary_c is stored. This options are not really used",
+    "_binary_c_config_executable": "Full path of the binary_c-config executable. This options is not used in the population object.",
+    "_binary_c_executable": "Full path to the binary_c executable. This options is not used in the population object.",
+    "_binary_c_shared_library": "Full path to the libbinary_c file. This options is not used in the population object",
+    "verbosity": "Verbosity of the population code. Default is 0, by which only errors will be printed. Higher values will show more output, which is good for debugging.",
+    "binary": "Set this to 1 if the population contains binaries. Input: int",  # TODO: write what effect this has.
+    "amt_cores": "The amount of cores that the population grid will use. The multiprocessing is useful but make sure to figure out how many logical cores the machine has. The core is multiprocessed, not multithreaded, and will gain no extra speed when amt_cores exceeds the amount of logical cores. Input: int",
+    "_start_time_evolution": "Variable storing the start timestamp of the population evolution. Set by the object itself.",  # TODO: make sure this is logged to a file
+    "_end_time_evolution": "Variable storing the end timestamp of the population evolution. Set by the object itself",  # TODO: make sure this is logged to a file
+    "_total_starcount": "Variable storing the total amount of systems in the generator. Used and set by the population object.",
+    "_custom_logging_shared_library_file": "filename for the custom_logging shared library. Used and set by the population object",
+    "_errors_found": "Variable storing a boolean flag whether errors by binary_c are encountered.",
+    "_errors_exceeded": "Variable storing a boolean flag whether the amount of errors was higher than the set threshold (failed_systems_threshold). If True, then the commandline arguments of the failing systems will not be stored in the failed_system_log files.",
+    "source_file_filename": "Variable containing the source file containing lines of binary_c commandline calls. These all have to start with binary_c.",  # TODO: Expand
+    "results": "Dictionary in which the user can place their results. This dictionary gets merged at the end of a mulitprocessing simulation.",
+    "C_auto_logging": "Dictionary containing parameters to be logged by binary_c. The structure of this dictionary is as follows: the key is used as the headline which the user can then catch. The value at that key is a list of binary_c system parameters (like star[0].mass)",
+    "C_logging_code": "Variable to store the exact code that is used for the custom_logging. In this way the user can do more complex logging, as well as putting these logging strings in files.",
+    "_failed_count": "Variable storing the amount of failed systems.",
+    "_evolution_type_options": "List containing the evolution type options.",
+    "_failed_prob": "Variable storing the total probability of all the failed systems",
+    "_failed_systems_error_codes": "List storing the unique error codes raised by binary_c of the failed systems",
+    "_grid_variables": "Dictionary storing the grid_variables. These contain properties which are accessed by the _generate_grid_code function",
+    "_population_id": "Variable storing a unique 32-char hex string.",
+    "_commandline_input": "String containing the arguments passed to the population object via the command line. Set and used by the population object.",
+    "_system_generator": "Function object that contains the system generator function. This can be from a grid, or a source file, or a montecarlo grid.",
+    "gridcode_filename": "Filename for the grid code. Set and used by the population object. TODO: allow the user to provide their own function, rather than only a generated function.",
+    "log_args": "Boolean to log the arguments. Unused ",  # TODO: fix the functionality for this and describe it properly
+    "log_args_dir": "Directory to log the arguments to. Unused",  # TODO: fix the functionality for this and describe it properly
+    "log_file": "Log file for the population object. Unused",  # TODO: fix the functionality for this and describe it properly
+    "custom_logging_func_memaddr": "Memory adress where the custom_logging_function is stored. Input: int",
+    "_count": "Counter tracking which system the generator is on.",
+    "_probtot": "Total probability of the population.",  # TODO: check whether this is used properly throughout
+    "_main_pid": "Main process ID of the master process. Used and set by the population object.",
+    "_store_memaddr": "Memory adress of the store object for binary_c.",
+    "failed_systems_threshold": "Variable storing the maximum amount of systems that are allowed to fail before logging their commandline arguments to failed_systems log files",
+    "parse_function": "Function that the user can provide to handle the output the binary_c. This function has to take the arguments (self, output). Its best not to return anything in this function, and just store stuff in the grid_options['results'] dictionary, or just output results to a file",
+    "condor": "Int flag whether to use a condor type population evolution.",  # TODO: describe this in more detail
+    "slurm": "Int flag whether to use a slurm type population evolution.",  # TODO: describe this in more detail
+    "population_type": "variable storing what kind of population type should be evolved. See population_type_options for the options.",  # TODO: make this functionality work properly
+    "_population_type_options": "List storing the population_type options.",
+    "_weight": "Weight factor for each system. The calculated probability is mulitplied by this. If the user wants each system to be repeated several times, then this variable should not be changed, rather change the _repeat variable instead, as that handles the reduction in probability per system. This is useful for systems that have a process with some random element in it.",  # TODO: add more info here, regarding the evolution splitting.
+    "_repeat": "Factor of how many times a system should be repeated. Consider the evolution splitting binary_c argument for supernovae kick repeating.",  # TODO: make sure this is used.
+    "evolution_type": "Variable containing the type of evolution used of the grid. Multiprocessing or linear processing",
+}
+
+#################################
+# Grid options functions
+
+# Utility functions
+def grid_options_help(option: str) -> dict:
+    """
+    Function that prints out the description of a grid option. Useful function for the user.
+
+    Args:
+        option: which option you want to have the description of
+
+    returns:
+        dict containg the option, the description if its there, otherwise empty string. And if the key doesnt exist, the dict is empty
+    """
+
+    option_keys = grid_options_defaults_dict.keys()
+    description_keys = grid_options_descriptions.keys()
+
+    if not option in option_keys:
+        print(
+            "Error: This is an invalid entry. Option does not exist, please choose from the following options:\n\t{}".format(
+                ", ".join(option_keys)
+            )
+        )
+        return {}
+
+    else:
+        if not option in description_keys:
+            print(
+                "This option has not been described properly yet. Please contact on of the authors"
+            )
+            return {option: ""}
+        else:
+            print(grid_options_descriptions[option])
+            return {option: grid_options_descriptions[option]}
+
+
+def grid_options_description_checker(print_info: bool = True) -> int:
+    """
+    Function that checks which descriptions are missing
+
+    Args:
+        print_info: whether to print out information about which options contain proper descriptions and which do not
+
+    Returns:
+        the amount of undescribed keys
+    """
+
+    # Get the keys
+    option_keys = grid_options_defaults_dict.keys()
+    description_keys = grid_options_descriptions.keys()
+
+    #
+    undescribed_keys = list(set(option_keys) - set(description_keys))
+
+    if undescribed_keys:
+        if print_info:
+            print(
+                "Warning: the following keys have no description yet:\n\t{}".format(
+                    ", ".join(sorted(undescribed_keys))
+                )
+            )
+            print(
+                "Total description progress: {:.2f}%%".format(
+                    100 * len(description_keys) / len(option_keys)
+                )
+            )
+    return len(undescribed_keys)
+
+
+def write_grid_options_to_rst_file(output_file: str) -> None:
+    """
+    Function that writes the descriptions of the grid options to a rst file
+
+    Tasks:
+        TODO: seperate things into private and public options
+
+    Args:
+        output_file: target file where the grid options descriptions are written to
+    """
+
+    # Get the options and the description
+    options = grid_options_defaults_dict
+    descriptions = grid_options_descriptions
+
+    # Get those that do not have a description
+    not_described_yet = list(set(options) - set(descriptions))
+
+    # separate public and private options
+    public_options = [key for key in options if not key.startswith("_")]
+    private_options = [key for key in options if key.startswith("_")]
+
+    # Check input
+    if not output_file.endswith(".rst"):
+        print("Filename doesn't end with .rst, please provide a proper filename")
+        return None
+
+    with open(output_file, "w") as f:
+        print("Population grid code options", file=f)
+        print("{}".format("=" * len("Population grid code options")), file=f)
+        print(
+            "The following chapter contains all grid code options, along with their descriptions",
+            file=f,
+        )
+        print(
+            "There are {} options that are not described yet.".format(
+                len(not_described_yet)
+            ),
+            file=f,
+        )
+        print("\n", file=f)
+
+        # Start public options part
+        print("Public options", file=f)
+        print("{}".format("-" * len("Public options")), file=f)
+        print("The following options are meant to be changed by the user.", file=f)
+        print("\n", file=f)
+
+        for public_option in sorted(public_options):
+            if public_option in descriptions:
+                print(
+                    "| **{}**: {}".format(public_option, descriptions[public_option]),
+                    file=f,
+                )
+            else:
+                print(
+                    "| **{}**: No description available yet".format(public_option),
+                    file=f,
+                )
+            print("", file=f)
+
+        # Start private options part
+        print("Private options", file=f)
+        print("{}".format("-" * len("Private options")), file=f)
+        print(
+            "The following options are not meant to be changed by the user, as these options are used and set internally by the object itself. The description still is provided, but just for documentation purposes.",
+            file=f,
+        )
+
+        for private_option in sorted(private_options):
+            if private_option in descriptions:
+                print(
+                    "| **{}**: {}".format(private_option, descriptions[private_option]),
+                    file=f,
+                )
+            else:
+                print(
+                    "| **{}**: No description available yet".format(private_option),
+                    file=f,
+                )
+            print("", file=f)
