@@ -1,68 +1,118 @@
 """
 Collection of useful functions.
 
-Part of this is copied/inspired by Robs
-Rob's binary_stars module
+Part of this is copied/inspired by Rob's binary_stars module
 
-Has functions to convert period to separation and vice versa.
-calc_period_from_sep($m1,$m2,$sep) calculate the period given the separation.
-calc_sep_from_period($m1,$m2,per) does the inverse.
-M1,M2,separation are in solar units, period in days.
-rzams($m,$z) gives you the ZAMS radius of a star
-ZAMS_collision($m1,$m2,$e,$sep,$z) returns 1 if stars collide on the ZAMS
+Functions:
+    - calc_period_from_sep(m1, m2, sep) calculate the period given the separation.
+    - calc_sep_from_period(m1, m2, per) does the inverse.
+    - rzams(m, z) gives you the ZAMS radius of a star
+    - ZAMS_collision(m1, m2, e, sep, z) returns 1 if stars collide on the ZAMS
+    - roche_lobe(q): returns roche lobe radius in units of separation
+    - ragb(m, z): radius at first thermal pulse
 
-# TODO: check whether these are correct
+Tasks:
+    - TODO: check whether these functions are correct
 """
+
 import math
+from typing import Union
 
 AURSUN = 2.150445198804013386961742071435e02
 YEARDY = 3.651995478818308811241877265275e02
 
 
-def calc_period_from_sep(M1, M2, sep):
+def calc_period_from_sep(
+    M1: Union[int, float], M2: Union[int, float], sep: Union[int, float]
+) -> Union[int, float]:
     """
     calculate period from separation
-    args : M1 (Msol), M2 (Msol), separation (Rsun)
 
-    returns the period (days)
+    Args:
+        M1: Primary mass in solar mass
+        M2: Secondary mass in solar mass
+        sep: Separation in solar radii
+
+    Returns:
+        period in years
     """
 
     return YEARDY * (sep / AURSUN) * math.sqrt(sep / (AURSUN * (M1 + M2)))
 
 
-def calc_sep_from_period(M1, M2, period):
+def calc_sep_from_period(
+    M1: Union[int, float], M2: Union[int, float], period: Union[int, float]
+) -> Union[int, float]:
     """
-    inverse of the above function
-    args : M1 (Msol), M2 (Msol), period (days)
+    Calculate separation from period.
 
-    returns the separation (Rsun)
+    TODO: check whether this is still correct
+
+    Args:
+        M1: Primary mass in solar mass
+        M2: Secondary mass in solar mass
+        period: Period of binary in days
+
+    Returns:
+        Separation in solar radii
     """
 
     return AURSUN * (period * period * (M1 + M2) / (YEARDY * YEARDY)) ** (1.0 / 3.0)
 
 
-def roche_lobe(q):
+def roche_lobe(q: Union[int, float]) -> Union[int, float]:
     """
     A function to evaluate R_L/a(q), Eggleton 1983.
+
+    # TODO: check the definition of the mass ratio
+    # TODO: check whether the logs are correct
+
+    Args:
+        q: mass ratio of the binary (secondary/primary)
+
+    Returns:
+        Roche lobe radius in units of the separation
     """
 
     p = q ** (1.0 / 3.0)
     return 0.49 * p * p / (0.6 * p * p + math.log(1.0 + p))
 
 
-def ragb(m, z):
+def ragb(m: Union[int, float], z: Union[int, float]) -> Union[int, float]:
     """
-    Function to calculate radius of a star at first thermal pulse as a function of mass (z=0.02)
+    Function to calculate radius of a star in units of solar radii at first thermal pulse as a function of mass (Z=0.02 only, but also good for Z=0.0001)
+
+    Args:
+        m: mass of star in units of solar mass
+        z: metallicity of star
+
+    Returns:
+        radius at first thermal pulse in units of solar radii
     """
-    # Z=0.02 only, but also good for Z=0.001
+
     return m * 40.0 + 20.0
-    # in Rsun
 
 
-def zams_collission(m1, m2, sep, e, z):
+def zams_collission(
+    m1: Union[int, float],
+    m2: Union[int, float],
+    sep: Union[int, float],
+    e: Union[int, float],
+    z: Union[int, float],
+) -> Union[int, float]:
     """
     given m1,m2, separation and eccentricity (and metallicity)
     determine if two stars collide on the ZAMS
+
+    Args:
+        m1: Primary mass in solar mass
+        m2: Secondary mass in solar mass
+        sep: separation in solar radii
+        e: eccentricity
+        z: metallicity
+
+    Returns:
+        integer boolean whether the binary stars will collide at pericenter
     """
 
     # calculate periastron distance
@@ -80,6 +130,15 @@ def zams_collission(m1, m2, sep, e, z):
 def rzams(m, z):
     """
     Function to determine the radius of a ZAMS star as a function of m and z:
+
+    Based on the fits of Tout et al., 1996, MNRAS, 281, 257
+
+    Args:
+        m: mass of star in solar mass
+        z: metallicity
+
+    Returns:
+        radius of star at ZAMS, in solar radii
     """
 
     lzs = math.log10(z / 0.02)
